""" powermon / protocols / pi30max.py """
import logging

from powermon.commands.result import ResultType
from powermon.protocols.pi30 import PI30
from powermon.commands.reading_definition import ResponseType
from powermon.commands.reading_definition import ReadingType

log = logging.getLogger("pi30max")

SOURCE_PRIORITY_LIST = ["Utility", "Solar first", "Solar + Utility", "Only Solar"]
OUTPUT_MODE_LIST = ["single machine",
                    "parallel",
                    "Phase 1 of 3 phase",
                    "Phase 2 of 3 phase",
                    "Phase 3 of 3 phase",
                    "Phase 1 of 2 phase",
                    "Phase 2 of 2 phase (120°)",
                    "Phase 2 of 2 phase (180°)"]
BATTERY_TYPE_LIST = ["AGM",
                     "Flooded",
                     "User",
                     "Pylontech",
                     "Shinheung",
                     "WECO",
                     "Soltaro",
                     "TBD",
                     "LIb-protocol compatible",
                     "3rd party Lithium"]

QUERY_COMMANDS = {
    "QSID": {
        "name": "QSID",
        "description": "Device Serial Number inquiry",
        "help": " -- queries the device serial number (length greater than 14)",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Serial Number",
                "reading_type": ReadingType.MESSAGE, "icon": "mdi:identifier",
                "response_type": ResponseType.TEMPLATE_BYTES, "format_template" : "r[2:int(r[0:2])+2]"}],
        "test_responses": [b"(1492932105105335005535\x94\x0e\r", ],
    },
    "QVFW3": {
        "name": "QVFW3",
        "description": "Remote CPU firmware version inquiry",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Remote CPU firmware version",
                "reading_type": ReadingType.MESSAGE, "icon": "mdi:identifier",
                "response_type": ResponseType.TEMPLATE_BYTES, "format_template": "r.removeprefix('VERFW:')"}],
        "test_responses": [b"(VERFW:00072.70\x53\xA7\r", ],
    },
    "VERFW": {
        "name": "VERFW",
        "description": "Bluetooth version inquiry",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Bluetooth firmware version",
                "reading_type": ReadingType.MESSAGE, "icon": "mdi:identifier",
                "response_type": ResponseType.TEMPLATE_BYTES, "format_template": "r.removeprefix('VERFW:')"}],
        "test_responses": [b"(VERFW:00072.70\x53\xA7\r", ],
    },
    "QFLAG": {
        "name": "QFLAG",
        "description": "Flag Status inquiry",
        "help": " -- queries the enabled / disabled state of various Inverter settings (e.g. buzzer, overload, interrupt alarm)",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Device Status",
                "reading_type": ReadingType.MULTI_ENABLE_DISABLE,
                "response_type": ResponseType.ENABLE_DISABLE_FLAGS,
                "options": {
                    "a": "Buzzer",
                    "b": "Overload Bypass",
                    "d": "Solar Feed to Grid",
                    "k": "LCD Reset to Default",
                    "u": "Overload Restart",
                    "v": "Over Temperature Restart",
                    "x": "LCD Backlight",
                    "y": "Primary Source Interrupt Alarm",
                    "z": "Record Fault Code",
                }}
        ],
        "test_responses": [
            b"(EakxyDbduvz\x8d\x73\r",
        ],
    },
    "QPIRI": {
        "name": "QPIRI",
        "description": "Current Settings inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "AC Input Voltage", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.VOLTS, "icon": "mdi:transmission-tower-import", "device_class": "voltage"},
            {"description": "AC Input Current", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.CURRENT, "icon": "mdi:current-ac", "device_class": "current"},
            {"description": "AC Output Voltage", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.VOLTS, "icon": "mdi:transmission-tower-export", "device_class": "voltage"},
            {"description": "AC Output Frequency", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.FREQUENCY, "icon": "mdi:current-ac", "device_class": "frequency"},
            {"description": "AC Output Current", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.CURRENT, "icon": "mdi:current-ac", "device_class": "current"},
            {"description": "AC Output Apparent Power", "response_type": ResponseType.INT,
                "reading_type": ReadingType.APPARENT_POWER, "icon": "mdi:power-plug", "device_class": "apparent_power"},
            {"description": "AC Output Active Power", "response_type": ResponseType.INT,
                "reading_type": ReadingType.WATTS, "icon": "mdi:power-plug", "device_class": "power", "state_class": "measurement"},
            {"description": "Battery Voltage", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.VOLTS, "icon": "mdi:battery-outline", "device_class": "voltage"},
            {"description": "Battery Recharge Voltage", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.VOLTS, "icon": "mdi:battery-outline", "device_class": "voltage"},
            {"description": "Battery Under Voltage", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.VOLTS, "icon": "mdi:battery-outline", "device_class": "voltage"},
            {"description": "Battery Bulk Charge Voltage", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.VOLTS, "icon": "mdi:battery-outline", "device_class": "voltage"},
            {"description": "Battery Float Charge Voltage", "response_type": ResponseType.FLOAT,
                "reading_type": ReadingType.VOLTS, "icon": "mdi:battery-outline", "device_class": "voltage"},
            {"description": "Battery Type", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": BATTERY_TYPE_LIST},
            {"description": "Max AC Charging Current", "reading_type": ReadingType.CURRENT, "response_type": ResponseType.INT, "icon": "mdi:current-ac", "device_class": "current"},
            {"description": "Max Charging Current", "reading_type": ReadingType.CURRENT, "response_type": ResponseType.INT, "icon": "mdi:current-ac", "device_class": "current"},
            {"description": "Input Voltage Range", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Appliance", "UPS"]},
            {"description": "Output Source Priority",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": [
                    "Utility Solar Battery",
                    "Solar Utility Battery",
                    "Solar Battery Utility"]},
            {"description": "Charger Source Priority",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": [
                    "Utility first",
                    "Solar first",
                    "Solar + Utility",
                    "Only solar charging permitted"]},
            {"description": "Max Parallel Units", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.INT},
            {"description": "Machine Type",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.OPTION, "options": {"00": "Grid tie", "01": "Off Grid", "10": "Hybrid"}},
            {"description": "Topology", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["transformerless", "transformer"]},
            {"description": "Output Mode", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": OUTPUT_MODE_LIST},
            {"description": "Battery Redischarge Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "PV OK Condition",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": [
                    "As long as one unit of inverters has connect PV, parallel system will consider PV OK",
                    "Only All of inverters have connect PV, parallel system will consider PV OK"]},
            {"description": "PV Power Balance",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": [
                    "PV input max current will be the max charged current",
                    "PV input max power will be the sum of the max charged power and loads power"]},
            {"description": "Max charging time for CV stage", "reading_type": ReadingType.TIME_MINUTES, "response_type": ResponseType.INT},
            {"description": "Operation Logic",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST, "options": ["Automatic mode", "On-line mode", "ECO mode"]},
            {"description": "Max discharging current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:current-ac", "device_class": "current",
                "response_type": ResponseType.INT},
        ],
        "test_responses": [
            b"(230.0 21.7 230.0 50.0 21.7 5000 4000 48.0 46.0 42.0 56.4 54.0 0 10 010 1 0 0 6 01 0 0 54.0 0 1\x6F\x7E\r",
        ],
    },
    "QPIGS": {
        "name": "QPIGS",
        "description": "General Status Parameters inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "AC Input Voltage",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:transmission-tower-export", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "AC Input Frequency", "reading_type": ReadingType.FREQUENCY,
                "response_type": ResponseType.FLOAT,
                "icon": "mdi:current-ac", "device_class": "frequency"},
            {"description": "AC Output Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT, "icon": "mdi:power-plug", "device_class": "voltage"},
            {"description": "AC Output Frequency",
                "reading_type": ReadingType.FREQUENCY, "icon": "mdi:current-ac", "device_class": "frequency",
                "response_type": ResponseType.FLOAT},
            {"description": "AC Output Apparent Power",
                "reading_type": ReadingType.APPARENT_POWER, "icon": "mdi:power-plug", "device_class": "apparent_power",
                "response_type": ResponseType.INT},
            {"description": "AC Output Active Power",
                "reading_type": ReadingType.WATTS, "icon": "mdi:power-plug", "device_class": "power", "state_class": "measurement",
                "response_type": ResponseType.INT},
            {"description": "AC Output Load", "reading_type": ReadingType.PERCENTAGE, "response_type": ResponseType.INT, "icon": "mdi:brightness-percent"},
            {"description": "BUS Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.INT, "icon": "mdi:details", "device_class": "voltage"},
            {"description": "Battery Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT, "icon": "mdi:battery-outline", "device_class": "voltage"},
            {"description": "Battery Charging Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:current-dc", "device_class": "current",
                "response_type": ResponseType.INT},
            {"description": "Battery Capacity", "reading_type": ReadingType.PERCENTAGE, "response_type": ResponseType.INT, "device_class": "battery"},
            {"description": "Inverter Heat Sink Temperature",
                "reading_type": ReadingType.TEMPERATURE, "icon": "mdi:details", "device_class": "temperature",
                "response_type": ResponseType.INT},
            {"description": "PV1 Input Current", "reading_type": ReadingType.CURRENT, "response_type": ResponseType.FLOAT, "icon": "mdi:solar-power", "device_class": "current"},
            {"description": "PV1 Input Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT, "icon": "mdi:solar-power", "device_class": "voltage"},
            {"description": "Battery Voltage from SCC",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:battery-outline", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "Battery Discharge Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:battery-negative", "device_class": "current",
                "response_type": ResponseType.INT},
            {"description": "Device Status",
                "reading_type": ReadingType.FLAGS,
                "response_type": ResponseType.FLAGS,
                "flags": [
                    "Is SBU Priority Version Added",
                    "Is Configuration Changed",
                    "Is SCC Firmware Updated",
                    "Is Load On",
                    "Is Battery Voltage to Steady While Charging",
                    "Is Charging On",
                    "Is SCC Charging On",
                    "Is AC Charging On",
                ]},
            {"description": "Battery Voltage Offset for Fans On (10mV)", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.INT},
            {"description": "EEPROM Version", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.INT},
            {"description": "PV1 Charging Power",
                "reading_type": ReadingType.WATTS, "icon": "mdi:solar-power", "device_class": "power", "state_class": "measurement",
                "response_type": ResponseType.INT},
            {"description": "Device Status2",
                "reading_type": ReadingType.FLAGS,
                "response_type": ResponseType.FLAGS, "flags": ["Is Charging to Float", "Is Switched On", "Is Dustproof Installed"]},
            {"description": "Solar Feed to Grid", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Disabled", "Enabled"]},
            {"description": "Country",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.OPTION,
                "options": {
                    "00": "India",
                    "01": "Germany",
                    "02": "South America",
                }},
            {"description": "Solar Feed to Grid Power",
                "reading_type": ReadingType.WATTS, "icon": "mdi:solar-power", "device_class": "power", "state_class": "measurement",
                "response_type": ResponseType.INT},
        ],
        "test_responses": [
            b"(227.2 50.0 230.3 50.0 0829 0751 010 447 54.50 020 083 0054 02.7 323.6 00.00 00000 00010110 00 00 00879 010\xf1\x8c\r",
            b"(227.2 50.0 230.3 50.0 0829 0751 010 447 54.50 020 083 0054 02.7 323.6 00.00 00000 00010110 00 00 00879 010 1 02 123\x1c\x84\r",
        ],
    },
    "QPIGS2": {
        "name": "QPIGS2",
        "description": "General Status Parameters inquiry 2",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "PV2 Input Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:solar-power", "device_class": "current",
                "response_type": ResponseType.FLOAT},
            {"description": "PV2 Input Voltage",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:solar-power", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "PV2 Charging Power",
                "reading_type": ReadingType.WATTS, "icon": "mdi:solar-power", "device_class": "power", "state_class": "measurement",
                "response_type": ResponseType.INT},
        ],
        "test_responses": [
            b"(03.1 327.3 01026 \xc9\x8b\r",
        ],
    },
    "QPGS": {
        "name": "QPGS",
        "description": "Parallel Information inquiry",
        "help": " -- example: QPGS0 queries the values of various metrics from instance 0 of parallel setup Inverters",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "Parallel instance number", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Not valid", "valid"]},
            {"description": "Serial number",
                "reading_type": ReadingType.MESSAGE, "icon": "mdi:identifier",
                "response_type": ResponseType.BYTES},
            {"description": "Work mode",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.OPTION,
                "options": {
                    "P": "Power On Mode",
                    "S": "Standby Mode",
                    "L": "Line Mode",
                    "B": "Battery Mode",
                    "F": "Fault Mode",
                    "H": "Power Saving Mode",
                    "D": "Shutdown Mode",
                    "Y": "Bypass"}},
            {"description": "Fault code",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.OPTION,
                "options": {
                    "00": "No fault",
                    "01": "Fan is locked",
                    "02": "Over temperature",
                    "03": "Battery voltage is too high",
                    "04": "Battery voltage is too low",
                    "05": "Output short circuited or Over temperature",
                    "06": "Output voltage is too high",
                    "07": "Over load time out",
                    "08": "Bus voltage is too high",
                    "09": "Bus soft start failed",
                    "10": "PV over current",
                    "11": "PV over voltage",
                    "12": "DC over current",
                    "13": "Battery discharge over current",
                    "51": "Over current inverter",
                    "52": "Bus voltage too low",
                    "53": "Inverter soft start failed",
                    "54": "Self-test failed",
                    "55": "Over DC voltage on output of inverter",
                    "56": "Battery connection is open",
                    "57": "Current sensor failed",
                    "58": "Output voltage is too low",
                    "60": "Power feedback protection",
                    "71": "Firmware version different",
                    "72": "Current sharing fault",
                    "80": "CAN communication failed",
                    "81": "Parallel host line lost",
                    "82": "Parallel synchronized signal lost",
                    "83": "Parallel battery voltage detect different",
                    "84": "AC input voltage or frequency detected different",
                    "85": "AC output current unbalanced",
                    "86": "AC output mode setting different"}},
            {"description": "Grid Voltage",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:power-plug", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "Grid Frequency",
                "reading_type": ReadingType.FREQUENCY, "icon": "mdi:current-ac", "device_class": "frequency",
                "response_type": ResponseType.FLOAT},
            {"description": "AC Output Voltage",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:power-plug", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "AC Output Frequency",
                "reading_type": ReadingType.FREQUENCY, "icon": "mdi:current-ac", "device_class": "frequency",
                "response_type": ResponseType.FLOAT},
            {"description": "AC Output Apparent Power",
                "reading_type": ReadingType.APPARENT_POWER, "icon": "mdi:power-plug", "device_class": "apparent_power",
                "response_type": ResponseType.INT},
            {"description": "AC Output Active Power",
                "reading_type": ReadingType.WATTS, "icon": "mdi:power-plug", "device_class": "power", "state_class": "measurement",
                "response_type": ResponseType.INT},
            {"description": "Load Percentage",
                "reading_type": ReadingType.PERCENTAGE, "icon": "mdi:brightness-percent",
                "response_type": ResponseType.INT},
            {"description": "Battery Voltage",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:battery-outline", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "Battery Charging Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:current-dc", "device_class": "current",
                "response_type": ResponseType.INT},
            {"description": "Battery Capacity",
                "reading_type": ReadingType.PERCENTAGE, "device_class": "battery",
                "response_type": ResponseType.INT},
            {"description": "PV1 Input Voltage",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:solar-power", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "Total Charging Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:brightness-percent", "device_class": "current",
                "response_type": ResponseType.INT},
            {"description": "Total AC Output Apparent Power",
                "reading_type": ReadingType.APPARENT_POWER, "icon": "mdi:power-plug", "device_class": "apparent_power",
                "response_type": ResponseType.INT},
            {"description": "Total Output Active Power",
                "reading_type": ReadingType.WATTS, "icon": "mdi:power-plug", "device_class": "power", "state_class": "measurement",
                "response_type": ResponseType.INT},
            {"description": "Total AC Output Percentage",
                "reading_type": ReadingType.PERCENTAGE, "icon": "mdi:brightness-percent",
                "response_type": ResponseType.INT},
            {"description": "Inverter Status",
                "reading_type": ReadingType.FLAGS,
                "response_type": ResponseType.FLAGS,
                "flags": [
                    "Is SCC OK",
                    "Is AC Charging",
                    "Is SCC Charging",
                    "Is Battery Over Voltage",
                    "Is Battery Under Voltage",
                    "Is Line Lost",
                    "Is Load On",
                    "Is Configuration Changed"]},
            {"description": "Output mode",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": OUTPUT_MODE_LIST},
            {"description": "Charger source priority",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST, "options": ["Utility first", "Solar first", "Solar + Utility", "Solar only"]},
            {"description": "Max Charger Current",
                "reading_type": ReadingType.CURRENT, "device_class": "current",
                "response_type": ResponseType.INT},
            {"description": "Max Charger Range",
                "reading_type": ReadingType.CURRENT, "device_class": "current", "response_type": ResponseType.INT},
            {"description": "Max AC Charger Current",
                "reading_type": ReadingType.CURRENT, "device_class": "current", "response_type": ResponseType.INT},
            {"description": "PV1 Input Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:solar-power", "device_class": "power",
                "response_type": ResponseType.INT},
            {"description": "Battery Discharge Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:battery-negative", "device_class": "current",
                "response_type": ResponseType.INT},
            {"description": "PV2 Input Voltage",
                "reading_type": ReadingType.VOLTS, "icon": "mdi:solar-power", "device_class": "voltage",
                "response_type": ResponseType.FLOAT},
            {"description": "PV2 Input Current",
                "reading_type": ReadingType.CURRENT, "icon": "mdi:solar-power", "device_class": "current",
                "response_type": ResponseType.INT},
        ],
        "test_responses": [
            b"(0 92932105105315 B 00 000.0 00.00 230.0 50.00 0989 0907 012 53.2 009 090 349.8 009 00989 00907 011 10100110 0 1 100 120 030 02 000 275.3 02i]\r",
        ],
        "regex": "QPGS(\\d+)$",
    },
    "QMOD": {
        "name": "QMOD",
        "description": "Mode inquiry",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Device Mode",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.OPTION,
                "options": {
                    "P": "Power on",
                    "S": "Standby",
                    "L": "Line",
                    "B": "Battery",
                    "F": "Fault",
                    "H": "Power Saving",
                    "D": "Shutdown",
                    "Y": "Bypass"}},
        ],
        "test_responses": [
            b"(S\xe5\xd9\r",
            b"(B\xe7\xc9\r",
        ],
    },
    "QPIWS": {
        "name": "QPIWS",
        "description": "Warning status inquiry",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Warning",
                "reading_type": ReadingType.FLAGS,
                "response_type": ResponseType.FLAGS,
                "flags": [
                    "PV loss warning",
                    "Inverter fault",
                    "Bus over fault",
                    "Bus under fault",
                    "Bus soft fail fault",
                    "Line fail warning",
                    "OPV short warning",
                    "Inverter voltage too low fault",
                    "Inverter voltage too high fault",
                    "Over temperature fault",
                    "Fan locked fault",
                    "Battery voltage to high fault",
                    "Battery low alarm warning",
                    "Reserved",
                    "Battery under shutdown warning",
                    "Battery derating warning",
                    "Overload fault",
                    "EEPROM fault",
                    "Inverter over current fault",
                    "Inverter soft fail fault",
                    "Self test fail fault",
                    "OP DC voltage over fault",
                    "Bat open fault",
                    "Current sensor fail fault",
                    "Battery short fault",
                    "Power limit warning",
                    "PV voltage high warning",
                    "MPPT overload fault",
                    "MPPT overload warning",
                    "Battery too low to charge warning",
                    "",
                    "Battery weak",
                    "Battery weak",
                    "Battery weak",
                    "",
                    "Battery equalisation warning"]}
        ],
        "test_responses": [
            b"(00000100000000001000000000000000\x56\xA6\r",
            b"(000000000000000000000000000000000000<\x8e\r",
        ],
    },
    "QDI": {
        "name": "QDI",
        "description": "Default Settings inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "AC Output Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "AC Output Frequency", "reading_type": ReadingType.FREQUENCY, "response_type": ResponseType.FLOAT},
            {"description": "Max AC Charging Current", "reading_type": ReadingType.CURRENT, "response_type": ResponseType.INT},
            {"description": "Battery Under Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "Battery Float Charge Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "Battery Bulk Charge Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "Battery Recharge Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "Max Charging Current", "reading_type": ReadingType.CURRENT, "response_type": ResponseType.INT},
            {"description": "Input Voltage Range", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Appliance", "UPS"]},
            {"description": "Output Source Priority",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST, "options": ["Utility first", "Solar first", "SBU first"]},
            {"description": "Charger Source Priority",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": [
                    "Utility first",
                    "Solar first",
                    "Solar + Utility",
                    "Only solar charging permitted"]},
            {"description": "Battery Type", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": BATTERY_TYPE_LIST},
            {"description": "Buzzer", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["enabled", "disabled"]},
            {"description": "Power saving", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "Overload restart", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "Over temperature restart", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "LCD Backlight", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "Primary source interrupt alarm", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "Record fault code", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "Overload bypass", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "LCD reset to default", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["disabled", "enabled"]},
            {"description": "Output mode", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": OUTPUT_MODE_LIST},
            {"description": "Battery Redischarge Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "PV OK condition",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": [
                    "As long as one unit of inverters has connect PV, parallel system will consider PV OK",
                    "Only All of inverters have connect PV, parallel system will consider PV OK"]},
            {"description": "PV Power Balance",
                "reading_type": ReadingType.MESSAGE,
                "response_type": ResponseType.LIST,
                "options": [
                    "PV input max current will be the max charged current",
                    "PV input max power will be the sum of the max charged power and loads power"]},
            {"description": "Max Charging Time at CV", "reading_type": ReadingType.TIME_MINUTES, "response_type": ResponseType.INT},
            {"description": "Max Discharging current", "reading_type": ReadingType.CURRENT, "response_type": ResponseType.INT},
        ],
        "test_responses": [
            b"(230.0 50.0 0030 44.0 54.0 56.4 46.0 60 0 0 2 0 0 0 0 0 1 1 1 0 1 0 54.0 0 1 224\xeb\xbc\r",
        ],
    },
    "QT": {
        "name": "QT",
        "description": "Device Time Inquiry",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [{"description": "Device Time", "reading_type": ReadingType.DATE_TIME, "response_type": ResponseType.BYTES}],
        "test_responses": [
            b"(20210726122606JF\r",
        ],
    },
    "QET": {
        "name": "QET",
        "description": "Total PV Generated Energy Inquiry",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Total PV Generated Energy",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:solar-power", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT}
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
    },
    "QEY": {
        "name": "QEY",
        "description": "Yearly PV Generated Energy Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "PV Generated Energy for Year",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:counter", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT},
            {"description": "Year",
                "reading_type": ReadingType.YEAR,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[3:])"},
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
        "regex": "QEY(\\d\\d\\d\\d)$",
    },
    "QEM": {
        "name": "QEM",
        "description": "Monthly PV Generated Energy Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "PV Generated Energy for Month",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:solar-power", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT},
            {"description": "Year",
                "reading_type": ReadingType.YEAR,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[3:7])"},
            {"description": "Month",
                "reading_type": ReadingType.MONTH,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "calendar.month_name[int(cn[7:])]"},
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
        "regex": "QEM(\\d\\d\\d\\d\\d\\d)$",
    },
    "QED": {
        "name": "QED",
        "description": "Daily PV Generated Energy Inquiry",
        "help": " -- display daily generated energy, format is QEDyyyymmdd",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "PV Generated Energy for Day",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:solar-power", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT},
            {"description": "Year",
                "reading_type": ReadingType.YEAR,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[3:7])"},
            {"description": "Month",
                "reading_type": ReadingType.MONTH,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "calendar.month_name[int(cn[7:9])]"},
            {"description": "Day",
                "reading_type": ReadingType.DAY,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[9:11])"},
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
        "regex": "QED(\\d\\d\\d\\d\\d\\d\\d\\d)$",
    },
    "QLT": {
        "name": "QLT",
        "description": "Total Output Load Energy Inquiry",
        "result_type": ResultType.SINGLE,
        "reading_definitions": [
            {"description": "Total Output Load Energy",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:counter", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT},
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
    },
    "QLY": {
        "name": "QLY",
        "description": "Yearly Output Load Energy Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "Output Load Energy for Year",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:counter", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT},
            {"description": "Year",
                "reading_type": ReadingType.YEAR,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[3:7])"},
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
        "regex": "QLY(\\d\\d\\d\\d)$",
    },
    "QLM": {
        "name": "QLM",
        "description": "Monthly Output Load Energy Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "Output Load Energy for Month",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:counter", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT},
            {"description": "Year",
                "reading_type": ReadingType.YEAR,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[3:7])"},
            {"description": "Month",
                "reading_type": ReadingType.MONTH,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "calendar.month_name[int(cn[7:9])]"},
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
        "regex": "QLM(\\d\\d\\d\\d\\d\\d)$",
    },
    "QLD": {
        "name": "QLD",
        "description": "Daily Output Load Energy Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "Output Load Energy for Day",
                "reading_type": ReadingType.WATT_HOURS, "icon": "mdi:counter", "device_class": "energy", "state_class": "total",
                "response_type": ResponseType.INT},
            {"description": "Year",
                "reading_type": ReadingType.YEAR,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[3:7])"},
            {"description": "Month",
                "reading_type": ReadingType.MONTH,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "calendar.month_name[int(cn[7:9])]"},
            {"description": "Day",
                "reading_type": ReadingType.DAY,
                "response_type": ResponseType.INFO_FROM_COMMAND, "format_template": "int(cn[9:11])"},
        ],
        "test_responses": [
            b"(00238800!J\r",
        ],
        "regex": "QLD(\\d\\d\\d\\d\\d\\d\\d\\d)$",
    },
    "QLED": {
        "name": "QLED",
        "description": "LED Status Parameters Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "LED Enabled", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Disabled", "Enabled"]},
            {"description": "LED Speed", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Low", "Medium", "Fast"]},
            {"description": "LED Effect", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Breathing", "Unknown", "Solid", "Right Scrolling"]},
            {"description": "LED Brightness", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.INT},
            {"description": "LED Number of Colors", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.INT},
            {"description": "RGB", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.BYTES},
        ],
        "test_responses": [
            b"(1 1 2 5 3 148000211255255255000255255\xdaj\r",
        ],
    },
    "QBEQI": {
        "name": "QBEQI",
        "description": "Battery Equalization Status Parameters Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "Equalization Enabled", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Disabled", "Enabled"]},
            {"description": "Equalization Time", "reading_type": ReadingType.TIME_MINUTES, "response_type": ResponseType.INT},
            {"description": "Equalization Period", "reading_type": ReadingType.TIME_DAYS, "response_type": ResponseType.INT},
            {"description": "Equalization Max Current", "reading_type": ReadingType.CURRENT, "response_type": ResponseType.INT},
            {"description": "Reserved1", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.BYTES},
            {"description": "Equalization Voltage", "reading_type": ReadingType.VOLTS, "response_type": ResponseType.FLOAT},
            {"description": "Reserved2", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.BYTES},
            {"description": "Equalization Over Time", "reading_type": ReadingType.TIME_MINUTES, "response_type": ResponseType.INT},
            {"description": "Equalization Active", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": ["Inactive", "Active"]},
            {"description": "Equalization Elasped Time", "reading_type": ReadingType.TIME_HOURS, "response_type": ResponseType.INT},
        ],
        "test_responses": [
            b"(1 030 030 080 021 55.40 224 030 0 0234y?\r",
        ],
    },
    "QOPPT": {
        "name": "QOPPT",
        "description": "Device Output Source Priority Time Order Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "Output Source Priority 00 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 01 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 02 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 03 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 04 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 05 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 06 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 07 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 08 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 09 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 10 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 11 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 12 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 13 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 14 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 15 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 16 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 17 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 18 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 19 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 20 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 21 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 22 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Output Source Priority 23 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Device Output Source Priority", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Selection of Output Source Priority Order 1",
                "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Selection of Output Source Priority Order 2",
                "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Selection of Output Source Priority Order 3",
                "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
        ],
        "test_responses": [
            b"(2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 2 1>>\r",
        ],
    },
    "QCHPT": {
        "name": "QCHPT",
        "description": "Device Charger Source Priority Time Order Inquiry",
        "result_type": ResultType.ORDERED,
        "reading_definitions": [
            {"description": "Charger Source Priority 00 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 01 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 02 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 03 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 04 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 05 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 06 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 07 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 08 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 09 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 10 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 11 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 12 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 13 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 14 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 15 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 16 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 17 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 18 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 19 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 20 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 21 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 22 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Charger Source Priority 23 hours", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Device Charger Source Priority", "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Selection of Charger Source Priority Order 1",
                "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Selection of Charger Source Priority Order 2",
                "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
            {"description": "Selection of Charger Source Priority Order 3",
                "reading_type": ReadingType.MESSAGE, "response_type": ResponseType.LIST, "options": SOURCE_PRIORITY_LIST},
        ],
        "test_responses": [
            b"(3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3 0 0 0\xd0\x8b\r",
        ],
    },
}

SETTER_COMMANDS = {
    "PLEDE": {
        "name": "PLEDE",
        "description": "Enable-Disable LED function",
        "help": " -- examples: PLEDE0 (disable LED), PLEDE1 (enable LED)",
        "regex": "PLEDE([01])$",
    },
    "PLEDS": {
        "name": "PLEDS",
        "description": "Set LED speed",
        "help": " -- examples: PLEDS0 (set LED speed low), PLEDS1 (set LED speed medium), PLEDS2 (set LED speed high)",
        "regex": "PLEDS([012])$",
    },
    "PLEDM": {
        "name": "PLEDM",
        "description": "Set LED effect",
        "help": " -- examples: PLEDM0 (set LED effect breathing), PLEDM2 (set LED effect solid), PLEDM3 (set LED right scrolling)",
        "regex": "PLEDM([0123])$",
    },
    "PLEDB": {
        "name": "PLEDB",
        "description": "Set LED brightness",
        "help": " -- examples: PLEDB1 (set LED brightness low), PLEDB5 (set LED brightness normal), PLEDB9 (set LED brightness high)",
        "regex": "PLEDB([123456789])$",
    },
    "PLEDT": {
        "name": "PLEDT",
        "description": "Set LED total number of colors",
        "help": " -- examples: PLEDT2 (set 2 LED colors), PLEDT3 (set 3 LED colors)",
        "regex": "PLEDT([123])$",
    },
    "PLEDC": {
        "name": "PLEDC",
        "description": "Set LED color",
        "help": " -- examples: PLEDCnRRRGGGBBB (n: 1 line mode, 2 AVR mode, 3 battery mode)",
        "regex": "PLEDC(\\d\\d\\d\\d\\d\\d\\d\\d\\d\\d)$",
    },
}

COMMANDS_TO_REMOVE = ["QVFW2"]


class PI30MAX(PI30):
    """ PI30 protocol handler for LV6048MAX and similar inverters """
    def __str__(self):
        return "PI30 protocol handler for LV6048MAX and similar inverters"

    def __init__(self) -> None:
        super().__init__()
        self.protocol_id = b"PI30MAX"
        self.add_command_definitions(QUERY_COMMANDS)
        self.add_command_definitions(SETTER_COMMANDS, result_type=ResultType.ACK)
        self.remove_command_definitions(COMMANDS_TO_REMOVE)
        self.check_definitions_count(expected=67)

        self.STATUS_COMMANDS = ["QPIGS", "QPIGS2"]
        self.SETTINGS_COMMANDS = ["QPIRI", "QFLAG"]
        self.DEFAULT_COMMAND = "QPI"
